// Copyright 2017 The Johns Hopkins University Applied Physics Laboratory.
// Licensed under the MIT License. See LICENSE.txt in the project root for full license information.

#pragma once

#include <pcl/point_cloud.h>
#include <eopcc/cloud.hpp>


namespace eopcc {

  template <typename T>
  void fill_pcl(pcl::PointCloud<T>& pcl_cloud, const PointCloud& pc, int start, int end) {
    pcl_cloud.width = end - start;
    pcl_cloud.height = 1;
    pcl_cloud.points.resize(pcl_cloud.width * pcl_cloud.height);

    double min_x = pc.extent.min_x;
    double min_y = pc.extent.min_y;
    double min_z = pc.extent.min_z;


    for (int i=0; i < (end - start); ++i) {
      pcl_cloud.points[i].x = pc.loc[i + start].x - min_x;
      pcl_cloud.points[i].y = pc.loc[i + start].y - min_y;
      pcl_cloud.points[i].z = pc.loc[i + start].z - min_z;
    }
  }

  template <typename T>
  void fill_pcl(pcl::PointCloud<T>& pcl_cloud, const PointCloud& pc) {
    fill_pcl(pcl_cloud, pc, 0, pc.size());
  }

  template <typename T>
  void fill_intensity(pcl::PointCloud<T>& pcl_cloud, const PointCloud& pc, int start, int end) {
    assert(pcl_cloud.width == end - start);
    assert(pcl_cloud.height == 1);


    for (int i=0; i < (end - start); ++i) {
      pcl_cloud.points[i].intensity = pc.intensity[i + start];
    }
  }

  template <typename T>
  void fill_intensity(pcl::PointCloud<T>& pcl_cloud, const PointCloud& pc) {
    fill_intensity(pcl_cloud, pc, 0, pc.size());
  }


  static void fill_pcl(pcl::PointCloud<pcl::PointXYZRGB>& pcl_cloud, const PointCloud& pc, int start, int end) {
    pcl_cloud.width = end - start;
    pcl_cloud.height = 1;
    pcl_cloud.points.resize(pcl_cloud.width * pcl_cloud.height);

    double min_x = pc.extent.min_x;
    double min_y = pc.extent.min_y;
    double min_z = pc.extent.min_z;

    for (int i=0; i < (end - start); ++i) {
      pcl_cloud.points[i].x = pc.loc[i + start].x - min_x;
      pcl_cloud.points[i].y = pc.loc[i + start].y - min_y;
      pcl_cloud.points[i].z = pc.loc[i + start].z - min_z;
      pcl_cloud.points[i].r = pc.color[i + start].red * 256;
      pcl_cloud.points[i].g = pc.color[i + start].green * 256;
      pcl_cloud.points[i].b = pc.color[i + start].blue * 256;
    }
  }


  template <typename T>
  void fill_pc(const pcl::PointCloud<T>& pcl_cloud, PointCloud& pc, int start, int end, const Extent& extent) {
    pc.resize(pcl_cloud.size());
    double min_x = extent.min_x;
    double min_y = extent.min_y;
    double min_z = extent.min_z;

    for (int i=0; i < (end - start); ++i) {
      pc.loc[i + start].x = pcl_cloud.points[i].x + min_x;
      pc.loc[i + start].y = pcl_cloud.points[i].y + min_y;
      pc.loc[i + start].z = pcl_cloud.points[i].z + min_z;
    }
  }

  template <typename T>
  void fill_pc_intensity(const pcl::PointCloud<T>& pcl_cloud, PointCloud& pc) {
    for (int i=0; i < pcl_cloud.size(); ++i) {
      pc.intensity[i] = pcl_cloud.points[i].intensity;
    }
  }

  template <typename T>
  void fill_pc(const pcl::PointCloud<T>& pcl_cloud, PointCloud& pc) {
    fill_pc(pcl_cloud, pc, 0, pcl_cloud.size(), pc.extent);
  }

  template <typename T>
  void fill_pc(const pcl::PointCloud<T>& pcl_cloud, PointCloud& pc, const Extent& extent) {
    fill_pc(pcl_cloud, pc, 0, pcl_cloud.size(), extent);
  }


  // Does not fill back RGB values to avoid lost precision
  template <typename T>
  void fill_pc(const pcl::PointCloud<T>& pcl_cloud, PointCloud& pc, int start, int end) {
    double min_x = pc.extent.min_x;
    double min_y = pc.extent.min_y;
    double min_z = pc.extent.min_z;

    for (int i=0; i < (end - start); ++i) {
      pc.loc[i + start].x = pcl_cloud.points[i].x + min_x;
      pc.loc[i + start].y = pcl_cloud.points[i].y + min_y;
      pc.loc[i + start].z = pcl_cloud.points[i].z + min_z;
    }
  }

} // end namespace eopcc
